@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2010-2017 Codership Oy <info@codership.com>
+// Copyright (C) 2010-2018 Codership Oy <info@codership.com>
 //
 
 #include "galera_common.hpp"
@@ -134,6 +134,7 @@ galera::ReplicatorSMM::ReplicatorSMM(const struct wsrep_init_args* args)
     safe_to_bootstrap_  (true),
     trx_params_         (config_.get(BASE_DIR), -1,
                          KeySet::version(config_.get(Param::key_format)),
+                         TrxHandle::Defaults.record_set_ver_,
                          gu::from_string<int>(config_.get(
                              Param::max_write_set_size))),
     uuid_               (WSREP_UUID_UNDEFINED),
@@ -340,8 +341,6 @@ wsrep_status_t galera::ReplicatorSMM::close()
 
 wsrep_status_t galera::ReplicatorSMM::async_recv(void* recv_ctx)
 {
-    assert(recv_ctx != 0);
-
     if (state_() == S_CLOSED || state_() == S_CLOSING)
     {
         log_error <<"async recv cannot start, provider in closed/closing state";
@@ -968,12 +967,19 @@ wsrep_status_t galera::ReplicatorSMM::post_rollback(TrxHandle* trx)
 
 wsrep_status_t galera::ReplicatorSMM::causal_read(wsrep_gtid_t* gtid)
 {
-    wsrep_seqno_t cseq(static_cast<wsrep_seqno_t>(gcs_.caused()));
+    wsrep_seqno_t cseq;
+    gu::datetime::Date wait_until(gu::datetime::Date::calendar() +
+                                  causal_read_timeout_);
 
-    if (cseq < 0)
+    try
     {
-        log_warn << "gcs_caused() returned " << cseq << " (" << strerror(-cseq)
-                 << ')';
+        gcs_.caused(cseq, wait_until);
+        assert(cseq >= 0);
+    }
+    catch (gu::Exception& e)
+    {
+        log_warn << "gcs_caused() returned " << -e.get_errno()
+                 << " (" << strerror(e.get_errno()) << ")";
         return WSREP_TRX_FAIL;
     }
 
@@ -986,8 +992,6 @@ wsrep_status_t galera::ReplicatorSMM::causal_read(wsrep_gtid_t* gtid)
         // at monitor drain and disallowing further waits until
         // configuration change related operations (SST etc) have been
         // finished.
-        gu::datetime::Date wait_until(gu::datetime::Date::calendar()
-                                      + causal_read_timeout_);
         if (gu_likely(co_mode_ != CommitOrder::BYPASS))
         {
             commit_monitor_.wait(cseq, wait_until);
@@ -1103,8 +1107,9 @@ writeset_from_handle (wsrep_po_handle_t& handle,
 //                gu::String<256>(trx_params.working_dir_) << '/' << &handle,
                 trx_params.working_dir_, wsrep_trx_id_t(&handle),
                 /* key format is not essential since we're not adding keys */
-                KeySet::version(trx_params.key_format_), NULL, 0,
-                0, WriteSetNG::MAX_VERSION, DataSet::MAX_VERSION, DataSet::MAX_VERSION,
+                KeySet::version(trx_params.key_format_), NULL, 0, 0,
+                trx_params.record_set_ver_,
+                WriteSetNG::MAX_VERSION, DataSet::MAX_VERSION, DataSet::MAX_VERSION,
                 trx_params.max_write_set_size_);
 
             handle.opaque = ret;
@@ -1285,6 +1290,8 @@ void galera::ReplicatorSMM::process_commit_cut(wsrep_seqno_t seq,
 
 void galera::ReplicatorSMM::establish_protocol_versions (int proto_ver)
 {
+    trx_params_.record_set_ver_ = gu::RecordSet::VER1;
+
     switch (proto_ver)
     {
     case 1:
@@ -1315,6 +1322,18 @@ void galera::ReplicatorSMM::establish_protocol_versions (int proto_ver)
         trx_params_.version_ = 3;
         str_proto_ver_ = 2;
         break;
+    case 8:
+        // Protocol upgrade to enforce 8-byte alignment in writesets.
+        trx_params_.version_ = 3;
+        trx_params_.record_set_ver_ = gu::RecordSet::VER2;
+        str_proto_ver_ = 2;
+        break;
+    case 9:
+        // Protocol upgrade to enable support for semi-shared key type.
+        trx_params_.version_ = 4;
+        trx_params_.record_set_ver_ = gu::RecordSet::VER2;
+        str_proto_ver_ = 2;
+        break;
     default:
         log_fatal << "Configuration change resulted in an unsupported protocol "
             "version: " << proto_ver << ". Can't continue.";
@@ -1520,7 +1539,7 @@ galera::ReplicatorSMM::process_conf_change(void*                    recv_ctx,
     else
     {
         // Non-primary configuration
-        if (state_uuid_ != WSREP_UUID_UNDEFINED)
+        if (state_uuid_ != WSREP_UUID_UNDEFINED && next_state == S_CLOSING)
         {
             st_.set (state_uuid_, STATE_SEQNO(), safe_to_bootstrap_);
         }
@@ -1860,9 +1879,12 @@ galera::ReplicatorSMM::update_state_uuid (const wsrep_uuid_t& uuid)
         *(const_cast<wsrep_uuid_t*>(&state_uuid_)) = uuid;
 
         std::ostringstream os; os << state_uuid_;
-
-        strncpy(const_cast<char*>(state_uuid_str_), os.str().c_str(),
-                sizeof(state_uuid_str_));
+        // Copy only non-nil terminated part of the source string
+        // and terminate the string explicitly to silence a warning
+        // generated by Wstringop-truncation
+        char* str(const_cast<char*>(state_uuid_str_));
+        strncpy(str, os.str().c_str(), sizeof(state_uuid_str_) - 1);
+        str[sizeof(state_uuid_str_) - 1] = '\0';
     }
 
     st_.set(uuid, WSREP_SEQNO_UNDEFINED, safe_to_bootstrap_);
