@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2010-2014 Codership Oy <info@codership.com>
+// Copyright (C) 2010-2018 Codership Oy <info@codership.com>
 //
 
 //! @file replicator_smm.hpp
@@ -88,11 +88,6 @@ namespace galera
             wsdb_.discard_conn_query(conn_id);
         }
 
-        void discard_local_conn(wsrep_conn_id_t conn_id)
-        {
-            wsdb_.discard_conn(conn_id);
-        }
-
         void apply_trx(void* recv_ctx, TrxHandle* trx);
 
         wsrep_status_t replicate(TrxHandle* trx, wsrep_trx_meta_t*);
@@ -354,8 +349,9 @@ namespace galera
                 case OOOC:
                     return true;
                 case LOCAL_OOOC:
-                    return trx_.is_local();
+                    if (trx_.is_local()) { return true; }
                     // in case of remote trx fall through
+                    // fall through
                 case NO_OOOC:
                     return (last_left + 1 == trx_.global_seqno());
                 }
@@ -497,7 +493,8 @@ namespace galera
          * |                 5 |           3 |              1 |               1 |
          * |                 6 |           3 |              2 |               1 |
          * |                 7 |           3 |              2 |               1 |
-         * |                 8 |           4 |              2 |               2 |
+         * |                 8 |           3 |              2 |               2 |
+         * |                 9 |           4 |              2 |               2 |
          * |--------------------------------------------------------------------|
          */
 
